/*
 * screenlock.c
 *
 * Copyright(C) 2006 Felix Bruns
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "pz.h"

extern int ti_widget_start(TWidget * wid);
extern TWidget * ti_new_standard_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));
extern TWidget * ti_new_password_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));

static PzModule *screenlock_module;
static PzConfig *screenlock_config;
static TWindow *screenlock_window;
static TWidget *screenlock_widget;
static ttk_surface image_lock;

static int screenlock_check_code(TWidget *this, char *input)
{
    const char *code = pz_get_string_setting(screenlock_config, 1);
    if(!strcmp(code, input)){
        pz_close_window(screenlock_window);
    }
    return 0;
}

static int screenlock_set_code(TWidget *this, char *input)
{
    pz_set_string_setting(screenlock_config, 1, input);
    pz_save_config(screenlock_config);
    pz_close_window(screenlock_window);
    return 0;
}

static void screenlock_draw(TWidget *this, ttk_surface srf)
{
    ttk_fillrect(srf, 0, 0, ttk_screen->w, ttk_screen->h, ttk_ap_getx("window.bg")->color);
    ttk_blit_image(image_lock, srf, ttk_screen->w/2-image_lock->w/2, (ttk_screen->h-ttk_screen->wy)/2-image_lock->h/2);
}

static int screenlock_button(TWidget *this, int button, int ms)
{
    switch(button){
    	case TTK_BUTTON_MENU:
            ttk_remove_widget(this->win, this);
            ttk_free_widget(this);
            break;
    	default:
            return ti_widget_start(this);
            break;
    }
    return TTK_EV_UNUSED;
}

static int screenlock_down(TWidget *this, int button) 
{
    TWidget *input_widget;
    switch(button){
    	case TTK_BUTTON_PREVIOUS:
    	case TTK_BUTTON_NEXT:
    	case TTK_BUTTON_MENU:
    	case TTK_BUTTON_PLAY:
    	case TTK_BUTTON_ACTION:
	    ttk_window_set_title(screenlock_window, "Enter Code...");
	    ttk_window_show_header(screenlock_window);
            input_widget = ti_new_password_text_widget(10, 10, screenlock_window->w-20, 10+ttk_text_height(ttk_textfont), 0, "", screenlock_check_code);
            input_widget->button = screenlock_button;
            ttk_add_widget(screenlock_window, input_widget);
            ti_widget_start(input_widget);
            return 0;
            break;
    }
    return TTK_EV_UNUSED;
}

static TWidget *new_screenlock_set_code_widget()
{
    screenlock_widget = ti_new_standard_text_widget(10, 10, screenlock_window->w-20, 10+ttk_text_height(ttk_textfont), 0, "", screenlock_set_code);

    return screenlock_widget;
}

static TWidget *new_screenlock_widget()
{
    screenlock_widget            = ttk_new_widget(0, 0);
    screenlock_widget->focusable = 1;
    screenlock_widget->dirty     = 1;
    screenlock_widget->draw      = screenlock_draw;
    screenlock_widget->down      = screenlock_down;

    return screenlock_widget;
}

static TWindow *new_screenlock_set_code_window()
{
    screenlock_window = pz_new_window("Set Code...", PZ_WINDOW_NORMAL);

    ttk_add_widget(screenlock_window, new_screenlock_set_code_widget());

    return pz_finish_window(screenlock_window);
}

static TWindow *new_screenlock_window()
{
    screenlock_window = pz_new_window("Screen Lock", PZ_WINDOW_NORMAL);

    image_lock = ttk_load_image(pz_module_get_datapath(screenlock_module, "lock.png"));

    ttk_add_widget(screenlock_window, new_screenlock_widget());

    return pz_finish_window(screenlock_window);
}

static void init_screenlock() 
{
    screenlock_module = pz_register_module("screenlock", NULL);
    screenlock_config = pz_load_config(pz_module_get_cfgpath(screenlock_module, "screenlock.conf"));
    pz_menu_add_action("/Extras/Utilities/Screen Lock/Lock", new_screenlock_window);
    pz_menu_add_action("/Extras/Utilities/Screen Lock/Set Code", new_screenlock_set_code_window);
}

PZ_MOD_INIT(init_screenlock)
